@@ -1,20 +1,38 @@
 '''Frame is the Client Framework that verifies and installs entries, and generates statistics'''
-__revision__ = '$Revision: 2546 $'
+__revision__ = '$Revision: 3012 $'
 
 import logging, time
 import Bcfg2.Client.Tools
 
+def cmpent(ent1, ent2):
+    '''Sort entries'''
+    if ent1.tag != ent2.tag:
+        return cmp(ent1.tag, ent2.tag)
+    else:
+        return cmp(ent1.get('name'), ent2.get('name'))
+
 def promptFilter(prompt, entries):
     '''Filter a supplied list based on user input'''
     ret = []
-    for entry in [entry for entry in entries]:
+    entries.sort(cmpent)
+    for entry in entries[:]:
+        if 'qtext' in entry.attrib:
+            iprompt = entry.get('qtext')
+        else:
+            iprompt = prompt % (entry.tag, entry.get('name'))
         try:
-            if raw_input(prompt % (entry.tag, entry.get('name'))) in ['y', 'Y']:
+            if raw_input(iprompt) in ['y', 'Y']:
                 ret.append(entry)
         except:
             continue
     return ret
 
+toolset_defs = {'rh': {'Service':'chkconfig', 'Package':'rpm'},
+                'debian': {'Service':'deb', 'Package':'deb'},
+                'nexenta': {'Service':'smf', 'Package':'deb'},
+                'solaris': {'Service':'smf', 'Package':'sysv'},
+                'gentoo': {'Service':'rc-update', 'Package':'ebuild'}}
+
 class Frame:
     '''Frame is the container for all Tool objects and state information'''
     def __init__(self, config, setup, times):
@@ -25,21 +43,41 @@ class Frame:
         self.tools = []
         self.states = {}
         self.whitelist = []
+        self.blacklist = []
         self.removal = []
         self.logger = logging.getLogger("Bcfg2.Client.Frame")
-        for tool in Bcfg2.Client.Tools.__all__[:]:
+        if self.setup['drivers']:
+            tools = []
+            for tcandidate in self.setup['drivers'].split(','):
+                if tcandidate not in Bcfg2.Client.Tools.drivers:
+                    self.logger.error("Tool driver %s is not available"\
+                                      % (tcandidate))
+                else:
+                    tools.append(tcandidate)
+        else:
+            tools = Bcfg2.Client.Tools.default[:]
+        tclass = {}
+        for tool in tools:
+            tool_class = "Bcfg2.Client.Tools.%s" % tool
             try:
-                tool_class = "Bcfg2.Client.Tools.%s" % tool
-                mod = __import__(tool_class, globals(), locals(), ['*'])
+                tclass[tool] = getattr(__import__(tool_class, globals(), locals(), ['*']),
+                                       tool)
             except ImportError:
                 continue
 
+        for tool in tclass.values():
             try:
-                self.tools.append(getattr(mod, tool)(self.logger, setup, config, self.states))
+                self.tools.append(tool(self.logger, setup, config, self.states))
             except Bcfg2.Client.Tools.toolInstantiationError:
                 continue
             except:
                 self.logger.error("Failed to instantiate tool %s" % (tool), exc_info=1)
+
+        for tool in self.tools[:]:
+            for conflict in getattr(tool, 'conflicts', []):
+                [self.tools.remove(item) for item in self.tools \
+                 if item.__name__ == conflict]
+
         self.logger.info("Loaded tool drivers:")
         self.logger.info([tool.__name__ for tool in self.tools])
         if not self.setup['dryrun']:
@@ -51,11 +89,28 @@ class Frame:
                     tool.InstallConfigFile(cfile)
         # find entries not handled by any tools
         problems = [entry for struct in config for entry in struct if entry not in self.handled]
+        if toolset_defs.has_key(config.get('toolset')):
+            tdefs = toolset_defs[config.get('toolset')]
+            for problem in problems[:]:
+                if tdefs.has_key(problem.tag):
+                    problem.set('type', tdefs[problem.tag])
+                    problems.remove(problem)
+
         if problems:
             self.logger.error("The following entries are not handled by any tool:")
             self.logger.error(["%s:%s:%s" % (entry.tag, entry.get('type'), \
                                              entry.get('name')) for entry in problems])
             self.logger.error("")
+        entries = [(entry.tag, entry.get('name')) for struct in config for entry in struct]
+        multi = []
+        for entry in entries[:]:
+            if entries.count(entry) > 1:
+                multi.append(entry)
+                entries.remove(entry)
+        if multi:
+            self.logger.debug("The following entries are included multiple times:")
+            self.logger.debug(["%s:%s" % entry for entry in multi])
+            
                     
     def __getattr__(self, name):
         if name in ['extra', 'handled', 'modified', '__important__']:
@@ -91,34 +146,61 @@ class Frame:
             elif self.setup['remove'] == 'packages':
                 self.removal = [entry for entry in self.extra if entry.tag == 'Package']
 
+        candidates = [entry for entry in self.states if not self.states[entry]]
+        self.whitelist = [entry for entry in self.states if not self.states[entry]]
         if self.setup['dryrun']:
-            updated = [entry for entry in self.states if not self.states[entry]]
-            if updated:
+            if self.whitelist:
                 self.logger.info("In dryrun mode: suppressing entry installation for:")
                 self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) for entry \
-                                  in updated])
+                                  in self.whitelist])
+                self.whitelist = []
             if self.removal:
                 self.logger.info("In dryrun mode: suppressing entry removal for:")
                 self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) for entry \
                                   in self.removal])
             self.removal = []
             return
-        elif self.setup['interactive']:
-            self.whitelist = promptFilter(prompt, [entry for entry in self.states \
-                                                   if not self.states[entry]])
-            self.removal = promptFilter(rprompt, self.removal)
-        elif self.setup['bundle']:
-            # only install entries in specified bundle
-            mbs = [bund for bund in self.config.findall('./Bundle') \
-                   if bund.get('name') == self.setup['bundle']]
-            if not mbs:
-                self.logger.error("Could not find bundle %s" % (self.setup['bundle']))
-                return
-            self.whitelist = [entry for entry in self.states if not self.states[entry] \
-                              and entry in mbs[0].getchildren()]
+        # Here is where most of the work goes
+        # first perform bundle filtering
+        if self.setup['bundle']:
+            bundles = [b for b in self.config.findall('./Bundle') \
+                       if b.get('name') == self.setup['bundle']]
+            self.whitelist = [e for e in self.whitelist if \
+                              True in [e in b for b in bundles]]
         else:
-            # all systems are go
-            self.whitelist = [entry for entry in self.states if not self.states[entry]]
+            bundles = self.config.getchildren()
+
+        # first process prereq actions
+        for bundle in bundles[:]:
+            if bundle.tag != 'Bundle':
+                continue
+            actions = [a for a in bundle.findall('./Action') \
+                       if a.get('timing') != 'post']
+            # now we process all "always actions"
+            bmodified = len([item for item in bundle if item in self.whitelist])
+            for action in actions:
+                if bmodified or action.get('when') == 'always':
+                    self.DispatchInstallCalls([action])
+            # need to test to fail entries in whitelist
+            if False in [self.states[a] for a in actions]:
+                # then display bundles forced off with entries
+                self.logger.info("Bundle %s failed prerequisite action" % \
+                                 (bundle.get('name')))
+                bundles.remove(bundle)
+                b_to_remv = [ent for ent in self.whitelist if ent in bundle]
+                if b_to_remv:
+                    self.logger.info("Not installing entries from Bundle %s" % \
+                                     (bundle.get('name')))
+                    self.logger.info(["%s:%s" % (e.tag, e.get('name')) for e in b_to_remv])
+                    [self.whitelist.remove(ent) for ent in b_to_remv]
+
+        if self.setup['interactive']:
+            self.whitelist = promptFilter(prompt, self.whitelist) 
+            self.removal = promptFilter(rprompt, self.removal) 
+
+        for entry in candidates:
+            if entry not in self.whitelist:
+                self.blacklist.append(entry)
 
     def DispatchInstallCalls(self, entries):
         '''Dispatch install calls to underlying tools'''
@@ -134,30 +216,41 @@ class Frame:
     def Install(self):
         '''Install all entries'''
         self.DispatchInstallCalls(self.whitelist)
+        mods = self.modified
+        mbundles = [struct for struct in self.config.findall('Bundle') if \
+                    [mod for mod in mods if mod in struct]]
+
         if self.modified:
             # Handle Bundle interdeps
-            mods = self.modified
-            mbundles = [struct for struct in self.config if struct.tag == 'Bundle' and \
-                        [mod for mod in mods if mod in struct]]
             if mbundles:
-                self.logger.info("The Following Bundles have been modifed:")
+                self.logger.info("The Following Bundles have been modified:")
                 self.logger.info([mbun.get('name') for mbun in mbundles])
                 self.logger.info("")
             tbm = [(t, b) for t in self.tools for b in mbundles]
             for tool, bundle in tbm:
                 try:
-                    tool.Inventory(bundle)
+                    tool.Inventory([bundle])
                 except:
                     self.logger.error("%s.Inventory() call failed:" % tool.__name__, exc_info=1)
             clobbered = [entry for bundle in mbundles for entry in bundle \
-                         if not self.states[entry]]
-            if not self.setup['interactive']:
-                self.DispatchInstallCalls(clobbered)
-            for tool, bundle in tbm:
+                         if not self.states[entry] and entry not in self.blacklist]
+            if clobbered:
+                self.logger.debug("Found clobbered entries:")
+                self.logger.debug(["%s:%s" % (entry.tag, entry.get('name')) \
+                                   for entry in clobbered])
+                if not self.setup['interactive']:
+                    self.DispatchInstallCalls(clobbered)
+
+        for bundle in self.config.findall('.//Bundle'):
+            for tool in self.tools:
                 try:
-                    tool.BundleUpdated(bundle)
+                    if bundle in mbundles:
+                        tool.BundleUpdated(bundle)
+                    else:
+                        tool.BundleNotUpdated(bundle)
                 except:
-                    self.logger.error("%s.BundleUpdated() call failed:" % (tool.__name__), exc_info=1)
+                    self.logger.error("%s.BundleNotUpdated() call failed:" % \
+                                      (tool.__name__), exc_info=1)
                 
     def Remove(self):
         '''Remove extra entries'''
@@ -174,8 +267,15 @@ class Frame:
         self.logger.info('\nPhase: %s' % phase)
         self.logger.info('Correct entries:\t%d' % self.states.values().count(True))
         self.logger.info('Incorrect entries:\t%d' % self.states.values().count(False))
+        if phase == 'final' and self.states.values().count(False):
+            self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) for \
+                              entry in self.states if not self.states[entry]])
         self.logger.info('Total managed entries:\t%d' % len(self.states.values()))
         self.logger.info('Unmanaged entries:\t%d' % len(self.extra))
+        if phase == 'final' and self.setup['extra']:
+            self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) \
+                              for entry in self.extra])
+                
         self.logger.info("")
 
         if ((self.states.values().count(False) == 0) and not self.extra):
@@ -222,7 +322,9 @@ class Frame:
                               ([entry for entry in self.states if not \
                                 self.states[entry]], "Bad")]:
             container = Bcfg2.Client.XML.SubElement(stats, ename)
-            [container.append(item) for item in data]
+            for item in data:
+                item.set('qtext', '')
+                container.append(item)
 
         timeinfo = Bcfg2.Client.XML.Element("OpStamps")
         feedback.append(stats)
