@@ -1,5 +1,5 @@
 '''Debian Init Support for Bcfg2'''
-__revision__ = '$Revision: 2531 $'
+__revision__ = '$Revision: 3008 $'
 
 import glob, os, re
 import Bcfg2.Client.Tools
@@ -10,6 +10,7 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
     __execs__ = ['/usr/sbin/update-rc.d']
     __handles__ = [('Service', 'deb')]
     __req__ = {'Service': ['name', 'status']}
+    __svcrestart__ = 'restart'
     svcre = re.compile("/etc/.*/[SK]\d\d(?P<name>\S+)")
 
     # implement entry (Verify|Install) ops
@@ -60,9 +61,8 @@ class DebInit(Bcfg2.Client.Tools.SvcTool):
         return [Bcfg2.Client.XML.Element('Service', name=name, type='deb') for name \
                 in extra]
 
-    def Remove(self, entries):
+    def Remove(self, _):
         '''Remove extra service entries'''
-        self.logger.info('Removing services:')
-        self.logger.info([entry.get('name') for entry in entries])
-        for entry in entries:
-            self.cmd.run("rm -f /etc/rc*.d/S??%s" % (entry.get('name')))
+        # Extra service removal is nonsensical
+        # Extra services need to be reflected in the config
+        return
