@@ -1,9 +1,9 @@
 '''All POSIX Type client support for Bcfg2'''
-__revision__ = '$Revision: 2537 $'
+__revision__ = '$Revision: 3036 $'
 
 from stat import S_ISVTX, S_ISGID, S_ISUID, S_IXUSR, S_IWUSR, S_IRUSR, S_IXGRP
 from stat import S_IWGRP, S_IRGRP, S_IXOTH, S_IWOTH, S_IROTH, ST_MODE, S_ISDIR
-from stat import S_IFREG, ST_UID, ST_GID, S_ISREG, S_IFDIR, S_ISLNK
+from stat import S_IFREG, ST_UID, ST_GID, S_ISREG, S_IFDIR, S_ISLNK, ST_MTIME
 
 import binascii, difflib, grp, os, pwd, xml.sax.saxutils
 import Bcfg2.Client.Tools
@@ -96,22 +96,53 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             self.logger.error('User/Group resolution failed for path %s' % (entry.get('name')))
             owner = 'root'
             group = 'root'
-        perms = oct(os.stat(entry.get('name'))[ST_MODE])[-4:]
+        finfo = os.stat(entry.get('name'))
+        perms = oct(finfo[ST_MODE])[-4:]
+        if entry.get('mtime', '-1') != '-1':
+            mtime = str(finfo[ST_MTIME])
+        else:
+            mtime = '-1'
         if ((owner == entry.get('owner')) and
             (group == entry.get('group')) and
-            (perms == entry.get('perms'))):
+            (perms == entry.get('perms')) and
+            (mtime == entry.get('mtime', '-1'))):
             return True
         else:
             if owner != entry.get('owner'):
                 entry.set('current_owner', owner)
                 self.logger.debug("%s %s ownership wrong" % (entry.tag, entry.get('name')))
+                nqtext = entry.get('qtext', '') + '\n'
+                nqtext += "%s owner wrong. is %s should be %s" % \
+                          (entry.get('name'), owner, entry.get('owner'))
+                entry.set('qtext', nqtext)
             if group != entry.get('group'):
                 entry.set('current_group', group)
                 self.logger.debug("%s %s group wrong" % (entry.tag, entry.get('name')))
+                nqtext = entry.get('qtext', '') + '\n'
+                nqtext += "%s group is %s should be %s" % \
+                          (entry.get('name'), group, entry.get('group'))
+                entry.set('qtext', nqtext)
             if perms != entry.get('perms'):
                 entry.set('current_perms', perms)
-                self.logger.debug("%s %s permissions wrong: are %s should be %s" %
+                self.logger.debug("%s %s permissions are %s should be %s" %
                                (entry.tag, entry.get('name'), perms, entry.get('perms')))
+                nqtext = entry.get('qtext', '') + '\n'
+                nqtext += "%s perms are %s should be %s" % \
+                          (entry.get('name'), perms, entry.get('perms'))
+                entry.set('qtext', nqtext)
+            if mtime != entry.get('mtime', '-1'):
+                entry.set('current_mtime', mtime)
+                self.logger.debug("%s %s mtime is %s should be %s" \
+                                  % (entry.tag, entry.get('name'), mtime,
+                                     entry.get('mtime')))
+                nqtext = entry.get('qtext', '') + '\n'
+                nqtext += "%s mtime is %s should be %s" % \
+                          (entry.get('name'), mtime, entry.get('mtime'))
+                entry.set('qtext', nqtext)
+            if entry.tag != 'ConfigFile':
+                nnqtext = entry.get('qtext')
+                nnqtext += '\nInstall %s %s: (y/N) ' % (entry.tag, entry.get('name'))
+                entry.set('qtext', nnqtext)
             return False
 
     def InstallDirectory(self, entry):
@@ -194,16 +225,30 @@ class POSIX(Bcfg2.Client.Tools.Tool):
 
         try:
             content = open(entry.get('name')).read()
-        except IOError:
-            # file does not exist
+        except IOError, error:
+            self.logger.error("Failed to read %s: %s" % (error.filename, error.strerror))
             return False
         contentStatus = content == tempdata
         if not contentStatus:
             diff = '\n'.join([x for x in difflib.ndiff(content.split('\n'), tempdata.split('\n'))])
             try:
-                entry.set("current_diff", xml.sax.saxutils.quoteattr(diff))
+                entry.set("current_diff", xml.sax.saxutils.escape(diff))
             except:
                 pass
+            udiff = '\n'.join([x for x in difflib.unified_diff(content.split('\n'), \
+                                                               tempdata.split('\n'))])
+            try:
+                eudiff = udiff.encode('ascii')
+            except:
+                eudiff = "Binary file: no diff printed"
+            nqtext = entry.get('qtext', '')
+            if nqtext:
+                nqtext += '\n'
+            nqtext += eudiff 
+            entry.set('qtext', nqtext)
+        qtxt = entry.get('qtext', '')
+        qtxt += "\nInstall ConfigFile %s: (y/N): " % (entry.get('name'))
+        entry.set('qtext', qtxt)
         return contentStatus and permissionStatus
 
     def InstallConfigFile(self, entry):
@@ -215,24 +260,35 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             try:
                 os.lstat(parent)
             except:
-                self.logger.debug('Creating parent path for config file %s' % (entry.get('name')))
-                for idx in xrange(len(parent.split('/')[:-1])):
-                    current = '/'+'/'.join(parent.split('/')[1:2+idx])
+                self.logger.debug('Creating parent path for config file %s' % \
+                                  (entry.get('name')))
+                current = '/'
+                for next in parent.split('/')[1:]:
+                    current += next + '/'
                     try:
                         sloc = os.lstat(current)
                         try:
                             if not S_ISDIR(sloc[ST_MODE]):
+                                self.logger.debug('%s is not a directory; recreating' \
+                                                  % (current))
                                 os.unlink(current)
                                 os.mkdir(current)
                         except OSError:
                             return False
                     except OSError:
                         try:
+                            self.logger.debug("Creating non-existent path %s" % current)
                             os.mkdir(current)
                         except OSError:
                             return False
 
         # If we get here, then the parent directory should exist
+        if entry.get("paranoid", False) and self.setup.get("paranoid", False):
+            bkupnam = entry.get('name').replace('/', '_')
+            if self.cmd.run("cp %s /var/cache/bcfg2/%s" % (entry.get('name'), bkupnam))[0]:
+                self.logger.error("Failed to create backup file for ConfigFile %s" % \
+                                  (entry.get('name')))
+                return False
         try:
             newfile = open("%s.new"%(entry.get('name')), 'w')
             if entry.get('encoding', 'ascii') == 'base64':
@@ -247,11 +303,20 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 os.chown(newfile.name, pwd.getpwnam(entry.get('owner'))[2],
                          grp.getgrnam(entry.get('group'))[2])
             except KeyError:
+                self.logger.error("Failed to chown %s to %s:%s" % \
+                                  (entry.get('name'), entry.get('owner'),
+                                   entry.get('group')))
                 os.chown(newfile.name, 0, 0)
             os.chmod(newfile.name, calcPerms(S_IFREG, entry.get('perms')))
-            if entry.get("paranoid", False) and self.setup.get("paranoid", False):
-                self.cmd.run("cp %s /var/cache/bcfg2/%s" % (entry.get('name')))
             os.rename(newfile.name, entry.get('name'))
+            if entry.get('mtime', '-1') != '-1':
+                try:
+                    os.utime(entry.get('name'), (int(entry.get('mtime')),
+                                                 int(entry.get('mtime'))))
+                except:
+                    self.logger.error("ConfigFile %s mtime fix failed" \
+                                      % (entry.get('name')))
+                    return False
             return True
         except (OSError, IOError), err:
             if err.errno == 13:
