@@ -1,10 +1,14 @@
 '''This contains all Bcfg2 Tool modules'''
-__revision__ = '$Revision: 2581 $'
+__revision__ = '$Revision: 3032 $'
 
-__all__ = ["APT", "Blast", "Chkconfig", "DebInit", "Encap", "Portage",
-           "PostInstall", "POSIX", "RPM", "RcUpdate", "SMF", "SYSV"]
+__all__ = ["Action", "APT", "Blast", "Chkconfig", "DebInit", "Encap",
+           "launchd", "Portage", "POSIX", "RPM", "RPMng", 'rpmtools',
+           "RcUpdate", "SMF", "SYSV", "Yum", "YUMng"]
 
-import os, popen2, stat, sys, Bcfg2.Client.XML
+drivers = [item for item in __all__ if item not in ['rpmtools']]
+default = [item for item in drivers if item not in ['RPMng', 'YUMng']]
+
+import os, popen2, stat, sys, Bcfg2.Client.XML, time
 
 class toolInstantiationError(Exception):
     '''This error is called if the toolset cannot be instantiated'''
@@ -38,21 +42,24 @@ class executor:
         self.logger.debug('> %s' % command)
 
         runpipe = readonlypipe(command, bufsize=16384)
-        output = ''
+        output = []
+        try:#macosx doesn't like this
+            runpipe.fromchild.flush()
+        except IOError:
+            pass
+        line = runpipe.fromchild.readline()
         cmdstat = -1
         while cmdstat == -1:
-            runpipe.fromchild.flush()
-            moreOutput = runpipe.fromchild.readline()
-            if len(moreOutput) > 0:                
-                self.logger.debug('< %s' % moreOutput[:-1])
-            output += moreOutput
+            while line:
+                if len(line) > 0:
+                    self.logger.debug('< %s' % line[:-1])
+                    output.append(line[:-1])
+                line = runpipe.fromchild.readline()
+            time.sleep(0.1)
             cmdstat = runpipe.poll()
-        for line in runpipe.fromchild.readlines():
-            if len(line) > 0:
-                self.logger.debug('< %s' % line[:-1])
-            output += line
-
-        return (cmdstat, [line for line in output.split('\n') if line])
+        output += [line[:-1] for line in runpipe.fromchild.readlines() \
+                   if line]
+        return (cmdstat, output)
 
 class Tool:
     '''All tools subclass this. It defines all interfaces that need to be defined'''
@@ -89,7 +96,11 @@ class Tool:
 
     def BundleUpdated(self, _):
         '''This callback is used when bundle updates occur'''
-        pass
+        return
+
+    def BundleNotUpdated(self, _):
+        '''This callback is used when a bundle is not updated'''
+        return
 
     def Inventory(self, structures=[]):
         '''Dispatch verify calls to underlying methods'''
@@ -100,7 +111,9 @@ class Tool:
                                 if self.canVerify(entry)]:
             try:
                 func = getattr(self, "Verify%s" % (entry.tag))
-                self.states[entry] = func(entry, self.buildModlist(entry, struct))
+                mods = self.buildModlist(entry, struct)
+                mods += [c.get('name') for c in entry.findall("Ignore")]
+                self.states[entry] = func(entry, mods)
             except:
                 self.logger.error(
                     "Unexpected failure of verification method for entry type %s" \
@@ -136,7 +149,7 @@ class Tool:
         if entry.tag != 'Package' or struct.tag != 'Bundle':
             return []
         return [sentry.get('name') for sentry in struct if sentry.tag in \
-                ['ConfigFile', 'SymLink', 'Directory']]
+                ['ConfigFile', 'SymLink', 'Directory', 'Permissions']]
 
     def canVerify(self, entry):
         '''test if entry has enough information to be verified'''
@@ -236,13 +249,15 @@ class PkgTool(Tool):
     def FindExtraPackages(self):
         '''Find extra packages'''
         packages = [entry.get('name') for entry in self.getSupportedEntries()]
-        extras = [key for key in self.installed if key not in packages]
-        return [Bcfg2.Client.XML.Element('Package', name=name, type=self.pkgtype) \
-                for name in extras]
+        extras = [data for data in self.installed.iteritems() if data[0] not in packages]
+        return [Bcfg2.Client.XML.Element('Package', name=name, \
+                                         type=self.pkgtype, version=version) \
+                                         for (name, version) in extras]
 
 class SvcTool(Tool):
     '''This class defines basic Service behavior'''
     __name__ = 'SvcTool'
+    __svcrestart__ = 'reload'
 
     def BundleUpdated(self, bundle):
         '''The Bundle has been updated'''
@@ -250,8 +265,10 @@ class SvcTool(Tool):
             if self.handlesEntry(entry):
                 if entry.get('status') == 'on':
                     self.logger.debug('Restarting service %s' % entry.get('name'))
-                    self.cmd.run('/etc/init.d/%s %s' % \
-                                 (entry.get('name'), entry.get('reload', 'reload')))
+                    rc = self.cmd.run('/etc/init.d/%s %s' % \
+                                      (entry.get('name'), entry.get('reload', self.__svcrestart__)))[0]
                 else:
                     self.logger.debug('Stopping service %s' % entry.get('name'))
-                    self.cmd.run('/etc/init.d/%s stop' %  (entry.get('name')))
+                    rc = self.cmd.run('/etc/init.d/%s stop' %  (entry.get('name')))[0]
+                if rc:
+                    self.logger.error("Failed to restart service %s" % (entry.get('name')))
