@@ -1,5 +1,5 @@
 '''Option parsing library for utilities'''
-__revision__ = '$Revision: 2345 $'
+__revision__ = '$Revision: 3027 $'
 
 import getopt, os, sys, ConfigParser
 # (option, env, cfpath, default value, option desc, boolean, arg desc)
@@ -70,19 +70,17 @@ class BasicOptionParser:
                 else:
                     ret[key] = sys.argv[sys.argv.index(option[0]) + 1]
                 continue
-            elif envvar and os.environ.has_key(envvar):
+            if envvar and os.environ.has_key(envvar):
                 ret[key] = os.environ[envvar]
                 continue
-            elif self.configfile and cfpath:
+            if self.configfile and cfpath:
                 try:
                     value = apply(cf.get, cfpath)
                     ret[key] = value
                     continue
                 except:
                     pass
-            else:
-                ret[key] = default
-                continue
+            ret[key] = default
         return ret
     
 class OptionParser(BasicOptionParser):
