@@ -1,10 +1,13 @@
 '''Cobalt component base classes'''
-__revision__ = '$Revision: 2533 $'
+__revision__ = '$Revision: 3049 $'
 
 import atexit, logging, select, signal, socket, sys, time, urlparse, xmlrpclib, cPickle, ConfigParser
-import BaseHTTPServer, OpenSSL.SSL, SimpleXMLRPCServer, SocketServer
+import BaseHTTPServer, SimpleXMLRPCServer, SocketServer
+import Bcfg2.tlslite.errors
+import Bcfg2.tlslite.api
 
 import Bcfg2.Client.Proxy as Proxy
+from Bcfg2.tlslite.TLSConnection import TLSConnection
 
 log = logging.getLogger('Component')
 
@@ -24,8 +27,6 @@ class CobaltXMLRPCRequestHandler(SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
             # get arguments
             data = self.rfile.read(int(self.headers["content-length"]))
             response = self.server._cobalt_marshalled_dispatch(data, self.client_address)
-        except OpenSSL.SSL.SysCallError:
-            log.error("Client %s unexpectedly closed connection" % (self.client_address[0]))
         except: # This should only happen if the module is buggy
             # internal error, report as HTTP server error
             log.error("Unexcepted handler failure in do_POST", exc_info=1)
@@ -33,62 +34,57 @@ class CobaltXMLRPCRequestHandler(SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
             self.end_headers()
         else:
             # got a valid XML RPC response
-            self.send_response(200)
-            self.send_header("Content-type", "text/xml")
-            self.send_header("Content-length", str(len(response)))
-            self.end_headers()
-            self.wfile.write(response)
-
-            # shut down the connection
-            self.wfile.flush()
-            self.connection.shutdown()
+            try:
+                self.send_response(200)
+                self.send_header("Content-type", "text/xml")
+                self.send_header("Content-length", str(len(response)))
+                self.end_headers()
+                self.wfile.write(response)
+                
+                # shut down the connection
+                self.wfile.flush()
+                #self.connection.shutdown()
+            except socket.error:
+                pass
 
     def setup(self):
+        '''Setup a working connection'''
         self.connection = self.request
         self.rfile = socket._fileobject(self.request, "rb", self.rbufsize)
         self.wfile = socket._fileobject(self.request, "wb", self.wbufsize)
 
-class SSLServer(BaseHTTPServer.HTTPServer):
-    '''This class encapsulates all of the ssl server stuff'''
+class TLSServer(Bcfg2.tlslite.api.TLSSocketServerMixIn,
+                BaseHTTPServer.HTTPServer):
+    '''This class is an tlslite-using SSLServer'''
     def __init__(self, address, keyfile, handler):
-        SocketServer.BaseServer.__init__(self, address, handler)
-        ctxt = OpenSSL.SSL.Context(OpenSSL.SSL.SSLv23_METHOD)
-        ctxt.use_privatekey_file (keyfile)
-        ctxt.use_certificate_file(keyfile)
-        #ctxt.load_verify_locations('/tmp/keys/CA.cert')
-        ctxt.set_verify(OpenSSL.SSL.VERIFY_PEER, self.verify_cb)
-        self.socket = OpenSSL.SSL.Connection(ctxt,
-                                             socket.socket(self.address_family, self.socket_type))
-        self.server_bind()
-        self.server_activate()
+        self.sc = Bcfg2.tlslite.api.SessionCache()
+        x509 = Bcfg2.tlslite.api.X509()
+        s = open(keyfile).read()
+        x509.parse(s)
+        self.key = Bcfg2.tlslite.api.parsePEMKey(s, private=True)
+        self.chain = Bcfg2.tlslite.api.X509CertChain([x509])
+        BaseHTTPServer.HTTPServer.__init__(self, address, handler)
 
-    def verify_cb(self, conn, cert, errnum, depth, ok):
-        '''handle cerificate verification'''
-        print "here"
-        print 'Got cert: %s' % (cert.get_subject())
-        print cert.get_pubkey()
-        return ok
+    def finish_request(self, sock, client_address):
+        sock.settimeout(90)
+        tlsConnection = TLSConnection(sock)
+        if self.handshake(tlsConnection) == True:
+            self.RequestHandlerClass(tlsConnection, client_address, self)
+            tlsConnection.close()
 
-    def handle_request(self):
-        """Handle one request, possibly blocking."""
+    def handshake(self, tlsConnection):
         try:
-            request, client_address = self.get_request()
+            tlsConnection.handshakeServer(certChain=self.chain,
+                                          privateKey=self.key,
+                                          sessionCache=self.sc)
+            tlsConnection.ignoreAbruptClose = True
+            return True
+        except Bcfg2.tlslite.errors.TLSError, error:
+            return False
         except socket.error:
-            return
-        if self.verify_request(request, client_address):
-            try:
-                self.process_request(request, client_address)
-            except Exception, err:
-                if isinstance(err, OpenSSL.SSL.Error):
-                    if isinstance(err, OpenSSL.SSL.SysCallError):
-                        log.error("Client %s unexpectedly closed connection" % (client_address[0]))
-                    else:
-                        log.error("%s from %s" % (err[0][0][2], client_address[0]))
-                else:
-                    log.error("Unknown socket I/O failure from %s" % (client_address[0]), exc_info=1)
-                self.close_request(request)
+            return False
                 
-class Component(SSLServer,
+class Component(TLSServer,
                 SimpleXMLRPCServer.SimpleXMLRPCDispatcher):
     """Cobalt component providing XML-RPC access"""
     __name__ = 'Component'
@@ -132,7 +128,7 @@ class Component(SSLServer,
         self.password = self.cfile.get('communication', 'password')
 
         try:
-            SSLServer.__init__(self, location, keyfile, CobaltXMLRPCRequestHandler)
+            TLSServer.__init__(self, location, keyfile, CobaltXMLRPCRequestHandler)
         except socket.error:
             self.logger.error("Failed to bind to socket")
             raise ComponentInitError
@@ -147,21 +143,22 @@ class Component(SSLServer,
         self.port = self.socket.getsockname()[1]
         self.url = "https://%s:%s" % (socket.gethostname(), self.port)
         self.logger.info("Bound to port %s" % self.port)
-        self.funcs.update({'HandleEvents':self.HandleEvents,
-                           'system.listMethods':self.addr_system_listMethods})
+        self.funcs.update({'system.listMethods':self.addr_system_listMethods})
         self.atime = 0
         self.assert_location()
         atexit.register(self.deassert_location)
 
-    def HandleEvents(self, address, event_list):
-        '''Default event handler'''
-        return True
-
     def _cobalt_marshalled_dispatch(self, data, address):
         """Decode and dispatch XMLRPC requests. Overloaded to pass through
         client address information
         """
-        rawparams, method = xmlrpclib.loads(data)
+        try:
+            rawparams, method = xmlrpclib.loads(data)
+        except:
+            self.logger.error("Failed to parse request from %s" \
+                              % (address[0]))
+            #open('/tmp/badreq', 'w').write(data)
+            return xmlrpclib.dumps(xmlrpclib.Fault(4, "Bad Request"))
         if len(rawparams) < 2:
             self.logger.error("No authentication included with request from %s" % address[0])
             return xmlrpclib.dumps(xmlrpclib.Fault(2, "No Authentication Info"))
@@ -170,7 +167,6 @@ class Component(SSLServer,
         params = rawparams[2:]
         # check authentication
         if not self._authenticate_connection(method, user, password, address):
-            self.logger.error("Authentication failure from %s" % address[0])
             return xmlrpclib.dumps(xmlrpclib.Fault(3, "Authentication Failure"))
         # generate response
         try:
