@@ -1,7 +1,7 @@
 '''This module provides the baseclass for Bcfg2 Server Plugins'''
-__revision__ = '$Revision: 2579 $'
+__revision__ = '$Revision: 3005 $'
 
-import logging, lxml.etree
+import logging, lxml.etree, re, copy
 
 from lxml.etree import XML, XMLSyntaxError
 
@@ -28,7 +28,7 @@ class Plugin(object):
       - Data collection (overloading GetProbes/ReceiveData)
     '''
     __name__ = 'Plugin'
-    __version__ = '$Id: Plugin.py 2579 2006-12-11 03:45:31Z desai $'
+    __version__ = '$Id: Plugin.py 3005 2007-04-03 12:47:24Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __rmi__ = []
 
@@ -59,6 +59,16 @@ class Plugin(object):
         '''This is the slow-path handler for configuration entry binding'''
         raise PluginExecutionError
 
+    def AcceptEntry(self, metadata, entry_type, entry_name, data):
+        '''This is the null per-plugin implementation
+        of bcfg2-admin pull'''
+        raise PluginExecutionError
+
+    def CommitChanges(self):
+        '''Handle revctl commits, if needed'''
+        # not implemented yet
+        pass
+
 # the rest of the file contains classes for coherent file caching
 
 class FileBacked(object):
@@ -89,6 +99,7 @@ class FileBacked(object):
 class DirectoryBacked(object):
     '''This object is a coherent cache for a filesystem hierarchy of files.'''
     __child__ = FileBacked
+    patterns = re.compile('.*')
 
     def __init__(self, name, fam):
         object.__init__(self)
@@ -109,10 +120,12 @@ class DirectoryBacked(object):
         if name == '':
             logger.info("got add for empty name")
         elif self.entries.has_key(name):
-            logger.info("got multiple adds for %s" % name)
+            self.entries[name].HandleEvent()
         else:
             if ((name[-1] == '~') or (name[:2] == '.#') or (name[-4:] == '.swp') or (name in ['SCCS', '.svn'])):
                 return
+            if not self.patterns.match(name):
+                return
             self.entries[name] = self.__child__('%s/%s' % (self.name, name))
             self.entries[name].HandleEvent()
 
@@ -187,8 +200,12 @@ class StructFile(XMLFileBacked):
                                          and not isinstance(item, lxml.etree._Comment)]
             for group in [item for item in worklist if item.tag == 'Group']:
                 # if only python had forceable early-binding
-                newpred = eval("lambda x:'%s' in x.groups and predicate(x)" % (group.get('name')),
-                               {'predicate':predicate})
+                if group.get('negate', 'false') == 'true':
+                    cmd = "lambda x:'%s' not in x.groups and predicate(x)"
+                else:
+                    cmd = "lambda x:'%s' in x.groups and predicate(x)"
+                    
+                newpred = eval(cmd % (group.get('name')), {'predicate':predicate})
                 work[newpred] = group.getchildren()
 
     def Match(self, metadata):
@@ -203,6 +220,8 @@ class INode:
     '''LNodes provide lists of things available at a particular group intersection'''
     raw = {'Client':"lambda x:'%s' == x.hostname and predicate(x)",
            'Group':"lambda x:'%s' in x.groups and predicate(x)"}
+    nraw = {'Client':"lambda x:'%s' != x.hostname and predicate(x)",
+            'Group':"lambda x:'%s' not in x.groups and predicate(x)"}
     containers = ['Group', 'Client']
     ignore = []
     
@@ -213,8 +232,13 @@ class INode:
             self.predicate = lambda x:True
         else:
             predicate = parent.predicate
-            if data.tag in self.raw.keys():
-                self.predicate = eval(self.raw[data.tag] % (data.get('name')), {'predicate':predicate})
+            if data.get('negate', 'false') == 'true':
+                psrc = self.nraw
+            else:
+                psrc = self.raw
+            if data.tag in psrc.keys():
+                self.predicate = eval(psrc[data.tag] % (data.get('name')),
+                                      {'predicate':predicate})
             else:
                 raise Exception
         mytype = self.__class__
@@ -291,7 +315,11 @@ class XMLSrc(XMLFileBacked):
             self.pnode.Match(metadata, cache[1])
             self.cache = cache
 
-class PrioDir(Plugin, DirectoryBacked):
+class XMLDirectoryBacked(DirectoryBacked):
+    '''Directorybacked for *.xml'''
+    patterns = re.compile('.*\.xml')    
+
+class PrioDir(Plugin, XMLDirectoryBacked):
     '''This is a generator that handles package assignments'''
     __name__ = 'PrioDir'
     __child__ = XMLSrc
@@ -299,14 +327,14 @@ class PrioDir(Plugin, DirectoryBacked):
     def __init__(self, core, datastore):
         Plugin.__init__(self, core, datastore)
         try:
-            DirectoryBacked.__init__(self, self.data, self.core.fam)
+            XMLDirectoryBacked.__init__(self, self.data, self.core.fam)
         except OSError:
             self.logger.error("Failed to load %s indices" % (self.__name__))
             raise PluginInitError
 
     def HandleEvent(self, event):
         '''Handle events and update dispatch table'''
-        DirectoryBacked.HandleEvent(self, event)
+        XMLDirectoryBacked.HandleEvent(self, event)
         for src in self.entries.values():
             for itype, children in src.items.iteritems():
                 for child in children:
@@ -338,7 +366,9 @@ class PrioDir(Plugin, DirectoryBacked):
             index = prio.index(max(prio))
 
         data = matching[index].cache[1][entry.tag][name]
-        [entry.attrib.__setitem__(key, data[key]) for key in data.keys()]
         if data.has_key('__text__'):
-            del entry.attrib['__text__']
             entry.text = data['__text__']
+        if data.has_key('__children__'):
+            [entry.append(copy.deepcopy(item)) for item in data['__children__']]
+        [entry.attrib.__setitem__(key, data[key]) for key in data.keys() \
+         if not key.startswith('__')]
