@@ -1,5 +1,5 @@
 '''This file provides the Hostbase plugin. It manages dns/dhcp/nis host information'''
-__revision__ = '$Revision: 2574 $'
+__revision__ = '$Revision: 3059 $'
 
 import sys, os
 os.environ['DJANGO_SETTINGS_MODULE'] = 'Bcfg2.Server.Hostbase.settings'
@@ -39,9 +39,9 @@ import re
 class Hostbase(Plugin):
     '''The Hostbase plugin handles host/network info'''
     __name__ = 'Hostbase'
-    __version__ = '$Id: Hostbase.py 2574 2006-12-08 21:34:03Z raffenet $'
+    __version__ = '$Id: Hostbase.py 3059 2007-04-23 19:37:19Z raffenet $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
-    filepath = '/etc/bind'
+    filepath = '/my/adm/hostbase/files/bind'
 
     def __init__(self, core, datastore):
 
@@ -61,6 +61,7 @@ class Hostbase(Plugin):
         self.templates = {'zone':loader.get_template('zone.tmpl'),
                           'reversesoa':loader.get_template('reversesoa.tmpl'),
                           'named':loader.get_template('named.tmpl'),
+                          'namedviews':loader.get_template('namedviews.tmpl'),
                           'reverseapp':loader.get_template('reverseappend.tmpl'),
                           'dhcp':loader.get_template('dhcpd.tmpl'),
                           'hosts':loader.get_template('hosts.tmpl'),
@@ -68,7 +69,10 @@ class Hostbase(Plugin):
                           }
         self.Entries['ConfigFile'] = {}
         self.__rmi__ = ['rebuildState']
-        self.rebuildState(None)
+        try:
+            self.rebuildState(None)
+        except:
+            raise PluginInitError
                  
     def FetchFile(self, entry, metadata):
         '''Return prebuilt file data'''
@@ -89,7 +93,7 @@ class Hostbase(Plugin):
                 if re.search('/etc/bind/', configfile):
                     SubElement(output, "ConfigFile", name=configfile)
         if metadata.hostname in self.dhcpservers:
-            SubElement(output, "ConfigFile", name="/etc/dhcpd.conf")
+            SubElement(output, "ConfigFile", name="/etc/dhcp3/dhcpd.conf")
         return [output]
 
     def rebuildState(self, _):
@@ -122,20 +126,12 @@ class Hostbase(Plugin):
                 serial = int(todaydate) * 100
             cursor.execute("""UPDATE hostbase_zone SET serial = \'%s\' WHERE id = \'%s\'""" % (str(serial), zone[0]))
 
-        cursor.execute("SELECT * FROM hostbase_zone WHERE zone <> \'.rev\'")
+        cursor.execute("SELECT * FROM hostbase_zone WHERE zone NOT LIKE \'%%.rev\'")
         zones = cursor.fetchall()
 
         iplist = []
         hosts = {}
 
-        cursor.execute("SELECT * FROM hostbase_zone WHERE zone = \'.rev\'")
-        reversezone = cursor.fetchall()[0]
-        cursor.execute("""SELECT n.name FROM hostbase_zone_nameservers z
-        INNER JOIN hostbase_nameserver n ON z.nameserver_id = n.id
-        WHERE z.zone_id = \'%s\'""" % reversezone[0])
-        reverse_nameservers = cursor.fetchall()
-
-
         for zone in zones:
             cursor.execute("""SELECT n.name FROM hostbase_zone_nameservers z
             INNER JOIN hostbase_nameserver n ON z.nameserver_id = n.id
@@ -156,74 +152,165 @@ class Hostbase(Plugin):
                 'mxs': mxs
                 })
             self.filedata[zone[1]] = self.templates['zone'].render(context)
+            self.filedata[zone[1] + '.external'] = self.templates['zone'].render(context)
 
             querystring = """SELECT h.hostname, p.ip_addr,
-            n.name, c.cname, m.priority, m.mx
+            n.name, c.cname, m.priority, m.mx, n.dns_view
             FROM (((((hostbase_host h INNER JOIN hostbase_interface i ON h.id = i.host_id)
             INNER JOIN hostbase_ip p ON i.id = p.interface_id)
             INNER JOIN hostbase_name n ON p.id = n.ip_id)
             INNER JOIN hostbase_name_mxs x ON n.id = x.name_id)
             INNER JOIN hostbase_mx m ON m.id = x.mx_id)
             LEFT JOIN hostbase_cname c ON n.id = c.name_id
-            WHERE h.hostname LIKE '%%%%%s' AND h.status = 'active'
+            WHERE n.name LIKE '%%%%%s'
+            AND h.status = 'active'
             ORDER BY h.hostname, n.name, p.ip_addr
             """ % zone[1]
             cursor.execute(querystring)
             zonehosts = cursor.fetchall()
             prevhost = (None, None, None, None)
+            cnames = ''
+            cnamesexternal = ''
             for host in zonehosts:
-                if not host[0].split(".", 1)[1] == zone[1]:
+                if not host[2].split(".", 1)[1] == zone[1]:
+                    self.filedata[zone[1]] += cnames
+                    self.filedata[zone[1] + '.external'] += cnamesexternal
+                    cnames = ''
+                    cnamesexternal = ''
                     continue
                 if not prevhost[1] == host[1] or not prevhost[2] == host[2]:
-                    self.filedata[zone[1]] += ("%-32s%-10s%-32s\n" %                   
-                                           (host[2].split(".", 1)[0], 'A', host[1]))
-                    self.filedata[zone[1]] += ("%-32s%-10s%-3s%s.\n" %                   
-                                           ('', 'MX', host[4], host[5]))
+                    self.filedata[zone[1]] += cnames
+                    self.filedata[zone[1] + '.external'] += cnamesexternal
+                    cnames = ''
+                    cnamesexternal = ''                    
+                    self.filedata[zone[1]] += ("%-32s%-10s%-32s\n" %
+                                               (host[2].split(".", 1)[0], 'A', host[1]))
+                    self.filedata[zone[1]] += ("%-32s%-10s%-3s%s.\n" %
+                                               ('', 'MX', host[4], host[5]))
+                    if host[6] == 'global':
+                        self.filedata[zone[1] + '.external'] += ("%-32s%-10s%-32s\n" %
+                                                                 (host[2].split(".", 1)[0], 'A', host[1]))
+                        self.filedata[zone[1] + '.external'] += ("%-32s%-10s%-3s%s.\n" %
+                                                                 ('', 'MX', host[4], host[5]))
+                elif not prevhost[5] == host[5]:
+                    self.filedata[zone[1]] += ("%-32s%-10s%-3s%s.\n" %
+                                               ('', 'MX', host[4], host[5]))                    
+                    if host[6] == 'global':
+                        self.filedata[zone[1] + '.external'] += ("%-32s%-10s%-3s%s.\n" %
+                                                                 ('', 'MX', host[4], host[5]))
+                        
                 if host[3]:
-                    if host[3].split(".", 1)[1] == zone[1]:
-                        self.filedata[zone[1]] += ("%-32s%-10s%-32s\n" %                   
-                                               (host[3].split(".", 1)[0],
-                                                'CNAME',host[2].split(".", 1)[0]))
-                    else:
-                        self.filedata[zone[1]] += ("%-32s%-10s%-32s\n" %                   
-                                               (host[3]+".",
-                                                'CNAME',
-                                                host[2].split(".", 1)[0]))
+                    try:
+                        if host[3].split(".", 1)[1] == zone[1]:
+                            cnames += ("%-32s%-10s%-32s\n" %
+                                                       (host[3].split(".", 1)[0],
+                                                        'CNAME',host[2].split(".", 1)[0]))
+                            if host[6] == 'global':
+                                cnamesexternal += ("%-32s%-10s%-32s\n" %
+                                                                         (host[3].split(".", 1)[0],
+                                                                          'CNAME',host[2].split(".", 1)[0]))
+                        else:
+                            cnames += ("%-32s%-10s%-32s\n" %
+                                                       (host[3]+".",
+                                                        'CNAME',
+                                                        host[2].split(".", 1)[0]))
+                            if host[6] == 'global':
+                                cnamesexternal += ("%-32s%-10s%-32s\n" %
+                                                                         (host[3]+".",
+                                                                          'CNAME',
+                                                                          host[2].split(".", 1)[0]))
+
+                    except:
+                        pass
                 prevhost = host
+            self.filedata[zone[1]] += cnames
+            self.filedata[zone[1] + '.external'] += cnamesexternal
             self.filedata[zone[1]] += ("\n\n%s" % zone[9])
-
+            self.filedata[zone[1] + '.external'] += ("\n\n%s" % zone[9])
             self.Entries['ConfigFile']["%s/%s" % (self.filepath, zone[1])] = self.FetchFile
+            self.Entries['ConfigFile']["%s/%s.external" % (self.filepath, zone[1])] = self.FetchFile
 
-
-        filelist = []
-        cursor.execute("""
-        SELECT ip_addr FROM hostbase_ip ORDER BY ip_addr
-        """)
-        three_subnet = [ip[0].rstrip('0123456789').rstrip('.') \
-                        for ip in cursor.fetchall()]
-        three_subnet_set = Set(three_subnet)
-        two_subnet = [subnet.rstrip('0123456789').rstrip('.') \
-                      for subnet in three_subnet_set]
-        two_subnet_set = Set(two_subnet)
-        filelist = [each for each in two_subnet_set \
-                    if two_subnet.count(each) > 1]
-        for each in three_subnet_set:
-            if each.rstrip('0123456789').rstrip('.') not in filelist:
-                filelist.append(each)
-
+        cursor.execute("SELECT * FROM hostbase_zone WHERE zone LIKE \'%%.rev\' AND zone <> \'.rev\'")
+        reversezones = cursor.fetchall()
+        
         reversenames = []
-        for filename in filelist:
-            towrite = filename.split('.')
-            towrite.reverse()
-            reversename = '.'.join(towrite)
+        for reversezone in reversezones:            
+            cursor.execute("""SELECT n.name FROM hostbase_zone_nameservers z
+            INNER JOIN hostbase_nameserver n ON z.nameserver_id = n.id
+            WHERE z.zone_id = \'%s\'""" % reversezone[0])
+            reverse_nameservers = cursor.fetchall()
+
             context = Context({
-                'inaddr': reversename,
+                'inaddr': reversezone[1].rstrip('.rev'),
                 'zone': reversezone,
                 'nameservers': reverse_nameservers,
                 })
 
-            self.filedata['%s.rev' % reversename] = self.templates['reversesoa'].render(context)
-            reversenames.append((reversename, filename))
+            self.filedata[reversezone[1]] = self.templates['reversesoa'].render(context)
+            self.filedata[reversezone[1] + '.external'] = self.templates['reversesoa'].render(context)
+            subnet = reversezone[1].split(".")
+            subnet.reverse()
+            reversenames.append((reversezone[1].rstrip('.rev'),".".join(subnet[1:]))) 
+
+        for filename in reversenames:
+            cursor.execute("""
+            SELECT DISTINCT h.hostname, p.ip_addr, n.dns_view FROM ((hostbase_host h
+            INNER JOIN hostbase_interface i ON h.id = i.host_id)
+            INNER JOIN hostbase_ip p ON i.id = p.interface_id)
+            INNER JOIN hostbase_name n ON n.ip_id = p.id
+            WHERE p.ip_addr LIKE '%s%%%%' AND h.status = 'active' ORDER BY p.ip_addr
+            """ % filename[1])
+            reversehosts = cursor.fetchall()
+            if len(filename[0].split(".")) == 2:
+                originlist = []
+                [originlist.append((".".join([ip[1].split(".")[2], filename[0]]),
+                                    ".".join([filename[1], ip[1].split(".")[2]])))
+                 for ip in reversehosts
+                 if (".".join([ip[1].split(".")[2], filename[0]]),
+                     ".".join([filename[1], ip[1].split(".")[2]])) not in originlist]
+                for origin in originlist:
+                    hosts = [(host[1].split("."), host[0])
+                             for host in reversehosts
+                             if host[1].rstrip('0123456789').rstrip('.') == origin[1]]
+                    hosts_external = [(host[1].split("."), host[0])
+                                     for host in reversehosts
+                                     if (host[1].rstrip('0123456789').rstrip('.') == origin[1]
+                                         and host[2] == 'global')]
+                    context = Context({
+                        'hosts': hosts,
+                        'inaddr': origin[0],
+                        'fileorigin': filename[0],
+                        })        
+                    self.filedata['%s.rev' % filename[0]] += self.templates['reverseapp'].render(context)
+                    context = Context({
+                        'hosts': hosts_external,
+                        'inaddr': origin[0],
+                        'fileorigin': filename[0],
+                        })        
+                    self.filedata['%s.rev.external' % filename[0]] += self.templates['reverseapp'].render(context)
+            else:
+                originlist = [filename[0]]
+                hosts = [(host[1].split("."), host[0])
+                         for host in reversehosts
+                         if (host[1].split("."), host[0]) not in hosts]
+                hosts_external = [(host[1].split("."), host[0])
+                                  for host in reversehosts
+                                  if ((host[1].split("."), host[0]) not in hosts
+                                  and host[2] == 'global')]
+                context = Context({
+                    'hosts': hosts,
+                    'inaddr': filename[0],
+                    'fileorigin': None,
+                    })        
+                self.filedata['%s.rev' % filename[0]] += self.templates['reverseapp'].render(context)
+                context = Context({
+                    'hosts': hosts_external,
+                    'inaddr': filename[0],
+                    'fileorigin': None,
+                    })
+                self.filedata['%s.rev.external' % filename[0]] += self.templates['reverseapp'].render(context)                
+            self.Entries['ConfigFile']['%s/%s.rev' % (self.filepath, filename[0])] = self.FetchFile
+            self.Entries['ConfigFile']['%s/%s.rev.external' % (self.filepath, filename[0])] = self.FetchFile
 
         ## here's where the named.conf file gets written
         context = Context({
@@ -231,47 +318,10 @@ class Hostbase(Plugin):
             'reverses': reversenames,
             })        
         self.filedata['named.conf'] = self.templates['named'].render(context)
-        self.Entries['ConfigFile']['%s/named.conf' % self.filepath] = self.FetchFile
+        self.Entries['ConfigFile']['/my/adm/hostbase/files/named.conf'] = self.FetchFile
+        self.filedata['named.conf.views'] = self.templates['namedviews'].render(context)
+        self.Entries['ConfigFile']['/my/adm/hostbase/files/named.conf.views'] = self.FetchFile
 
-        reversenames.sort()
-        for filename in reversenames:
-            print filename
-            originlist = []
-            cursor.execute("""
-            SELECT h.hostname, p.ip_addr FROM ((hostbase_host h
-            INNER JOIN hostbase_interface i ON h.id = i.host_id)
-            INNER JOIN hostbase_ip p ON i.id = p.interface_id)
-            WHERE p.ip_addr LIKE '%s%%%%' AND h.status = 'active' ORDER BY p.ip_addr
-            """ % filename[1])
-            reversehosts = cursor.fetchall()
-            if len(filename[0].split(".")) == 2:
-                [originlist.append((".".join([ip[1].split(".")[2], filename[0]]),
-                                    ".".join([filename[1], ip[1].split(".")[2]])))
-                 for ip in reversehosts
-                 if (".".join([ip[1].split(".")[2], filename[0]]),
-                     ".".join([filename[1], ip[1].split(".")[2]])) not in originlist]
-                for origin in originlist:
-                    hosts = [(host[1].split("."), host[0].split(".", 1))
-                             for host in reversehosts
-                             if host[1].rstrip('0123456789').rstrip('.') == origin[1]]
-                    context = Context({
-                        'hosts': hosts,
-                        'inaddr': origin[0],
-                        'fileorigin': filename[0],
-                        })        
-                    self.filedata['%s.rev' % filename[0]] += self.templates['reverseapp'].render(context)
-            else:
-                originlist = [filename[0]]
-                hosts = [(host[1].split("."), host[0].split(".", 1))
-                         for host in reversehosts]
-                context = Context({
-                    'hosts': hosts,
-                    'inaddr': filename[0],
-                    'fileorigin': None,
-                    })        
-                self.filedata['%s.rev' % filename[0]] += self.templates['reverseapp'].render(context)
-            print hosts
-            self.Entries['ConfigFile']['%s/%s.rev' % (self.filepath, filename[0])] = self.FetchFile
 
     def buildDHCP(self):
         '''Pre-build dhcpd.conf and stash in the filedata table'''
@@ -318,7 +368,7 @@ class Hostbase(Plugin):
             })
 
         self.filedata['dhcpd.conf'] = self.templates['dhcp'].render(context)
-        self.Entries['ConfigFile']['/etc/dhcp3/dhcpd.conf'] = self.FetchFile
+        self.Entries['ConfigFile']['/my/adm/hostbase/files/dhcpd.conf'] = self.FetchFile
 
 
     def buildHosts(self):
@@ -354,10 +404,11 @@ class Hostbase(Plugin):
             INNER JOIN hostbase_ip p ON i.id = p.interface_id)
             INNER JOIN hostbase_name n ON p.id = n.ip_id)
             LEFT JOIN hostbase_cname c ON n.id = c.name_id
-            WHERE p.ip_addr LIKE \'%s%%%%\' AND h.status = 'active'
-            ORDER BY p.ip_addr""" % three_octet[0]
+            WHERE p.ip_addr LIKE \'%s.%%%%\' AND h.status = 'active'""" % three_octet[0]
             cursor.execute(querystring)
-            append_data.append((three_octet, cursor.fetchall()))
+            tosort = list(cursor.fetchall())
+            tosort.sort(lambda x, y: cmp(int(x[2].split(".")[-1]), int(y[2].split(".")[-1])))
+            append_data.append((three_octet, tuple(tosort)))
 
         two_octets = [ip.rstrip('0123456789').rstrip('.') for ip in three_octets]
         two_octets_set = Set(two_octets)
@@ -394,7 +445,7 @@ class Hostbase(Plugin):
                 else:
                     if appenddata[0] == ip[0]:
                         simple = False
-                    ips.append((appenddata[2], appenddata[0], namelist,
+                    ips.append((appenddata[2], appenddata[0], Set(namelist),
                                 cnamelist, simple, appenddata[1]))
                     appenddata = ip
                     simple = True
@@ -403,7 +454,7 @@ class Hostbase(Plugin):
                     if ip[4]:
                         cnamelist.append(ip[4].split('.', 1)[0])
                         simple = False
-            ips.append((appenddata[2], appenddata[0], namelist,
+            ips.append((appenddata[2], appenddata[0], Set(namelist),
                         cnamelist, simple, appenddata[1]))
             context = Context({
                 'subnet': subnet[0],
@@ -533,7 +584,7 @@ olivia.ctd.anl.gov\n\n"""
             for each in netgroups[netgroup]:
                 fileoutput += each + "\n"
             self.filedata['%s-machines' % netgroup] = fileoutput
-            self.Entries['ConfigFile']['%s-machines' % netgroup] = self.FetchFile
+            self.Entries['ConfigFile']['/my/adm/hostbase/makenets/machines/%s-machines' % netgroup] = self.FetchFile
 
         cursor.execute("""
         UPDATE hostbase_host SET dirty=0
